@@ -66,8 +66,6 @@ class TestUtils(unittest.TestCase):
         data = load("data/sample.*")
         print(len(data))
         self.assertEquals(len(data), 6)
-        data = load("/Users/chenhao/workspace/XAgents/knowledge_base/test_service/config.json")
-        print(data)
 
     def test_batch_process_with_save(self):
         data = range(20)
